
#header {
    background: #FAF8A3 none repeat scroll 0% 0%;
}

div.devtools {
    position: fixed;
    float: right;
    right: 0px;
    bottom: 0px;
    background: url(++resource++box-bottom.png) bottom left no-repeat;
    width: 200px;
    z-index: 99;
}

div.devtools .header {
    font-size: small;
    padding: 4px;
}

div.devtools .header h4 {
    margin: 0;
    padding: 0em 1em 0em 2em;
    font-size: small;
}

div.devtools .body {
    padding-left: 1px;
    border-top: 1px #bab5ab solid;
    border-bottom: 1px #bab5ab solid;
    margin-bottom: 1ex;
}

div.devtools .body a {
    display: block;
    padding: 2px 1em;
    color: #304D6B;
    font-weight: bold;
    text-decoration: none;

    background: #eae8e3;
    border-top: 1px #eae8e3 solid;
    border-bottom: 1px #eae8e3 solid;
}

div.devtools .body a:hover {
    background: #7590ae;
    border-top: 1px #314e6c solid;
    border-bottom: 1px #314e6c solid;
    color: #fff !important;
}
