div.no-batch-navigation,
div.batch-navigation {
    font-size: 12px;
    line-height: 16px;
    margin-top: 16px;
    padding: 4px 8px;
}

.batch-navigation .batch-extra-navigation {
    float: right;
}

.batch-navigation span.previous,
.batch-navigation span.current,
.batch-navigation span.next,
.batch-navigation span.batch_page {
    margin: 0px;
    padding: 0px;
}

.batch-navigation span.batch_separator {
    margin-left: -4px;
    padding: 4px;
    cursor: default;
}

.batch-navigation span.batch_page a {
    margin-left: -4px;
    padding: 4px 8px 4px 8px;
}

.batch-navigation a.less {
    display: none;
}

.batch-navigation a.all {
    margin-left: 16px;
}

.batch-navigation a.previous,
.batch-navigation span.previous {
    margin-right:  8px;
}

.batch-navigation a.batch_page,
.batch-navigation a.batch_page:hover {
    text-decoration: none;
}
