/*
 * Stylesheet for SchoolTool
 *
 * Aesthetic styling
 *
 */


ul {
    list-style: none;
    padding-left: 1em;
}

input.disabled {
    color: #eae8e3 !important;
    background: #bab5ab !important;
    border: #eae8e3 !important;
}

td.number {
    text-align: right;
}

.section {
    margin: 0.5em;
    background: #eae8e3;
    padding: 2px;
    border: 1px #999 solid;
}

.instructor {
    float: left;
    text-align: center;
}

.instructor img {
    background: #d6d6d6;
    padding: 2px;
    border: 1px #999 solid;
}

/* This overrides schoolbell's style.css */
#header .logo {
    height: 60px;
    padding: 5px;
    vertical-align: middle;
}

#header > span#header-selected {

  background: white;
  color: #990000;
  font-weight: bold;
  border: 1px solid #990000;

}

/* Terms */

table.schooldaymodel {
    width: 90%;
    margin-bottom: 0.5ex;
}

table.schooldaymodel tr.legend td {
    text-align: center;
}
table.schooldaymodel tr.legend td.legend {
    text-align: left;
    padding-left: 0;
}
table.schooldaymodel td.holiday {
    color: red;
    background: #eeeeee;
    font-weight: bold;
}
table.schooldaymodel td.schoolday {
    color: black;
    background: #eeeeee;
    font-weight: normal;
}
table.schooldaymodel td {
    background: transparent;
    text-align: right;
    padding: 0ex 1ex;
}
table.schooldaymodel td label {
    display: inline;
    font-weight: inherit;
    float: none;
    width: auto;
    color: inherit;
}

/* Forms beautification for ttschema wizard */
span.hint,
p.hint {
    font-size: 80%;
    font-style: italic;
    color: #bab5ab;
}

/* Help clearing the mess generated by the field-names of a form which are
   floating and can break the layout of the whole page.
   For more information:
   http://www.complexspiral.com/publications/containing-floats/ */
div.clear {
    display: block;
    clear: left;
    margin: 0;
    line-height: 0;
    height: 0;
    visibility: hidden;
}


div.label {
    float: left;
    width: 10em;
    padding-top: 0.5ex;
}

div.value {
    margin-left: 10em;
}

table.timetable input.text {
    width: 7em;
}

ul.container-content a {
    text-decoration: none;
}

ul.container-content {
    padding: 0;
    margin: 0;
    margin: 0.25em 0 0 0.5em;
}

li.course {
    border: none;
    border-left: 1px #d6d6d6 solid;
    margin-bottom: 1em;
}

ul.section-list {
    margin: 0.25em 0 0 0.5em;
}

ul.section-list a {
    font-weight: normal;
}

.info-block ul {
    list-style: none;
}

.info-block li ul {
    border: none;
    border-left: 1px #d6d6d6 solid;
    padding: 0em 0em 0.5em 0.5em;
}

.info-block li.group a {
    font-weight: bold;
}

.info-block li.individual a {
    font-weight: normal;
}


.info-block li.item {
    list-style-type: square;
}

a#selected {
   background: #cccccc;
}

/* timetable display */

table.timetable {
    width: 90%;
}

table.timetable th {
    background: #cccccc;
}
table.timetable th.period {
    background: #ccccff;
}
table.timetable td {
    background: #eeeeee;
}

/* Calendar overlay */

#portlet-calendar-overlay a.go {
    top: 0em;
}

#portlet-calendar-overlay table.overlays {
    padding: none;
    margin: none;
    border-collapse: collapse;
}

#portlet-calendar-overlay table.overlays th {
    text-align: right;
}

#portlet-calendar-overlay table.overlays acronym {
    border-bottom: none;
}

/* Timetable setup form */

table.timetable_setup {
    border-collapse: collapse;
}
table.timetable_setup tr.conflict {
    background: yellow;
}
table.timetable_setup th,
table.timetable_setup td {
    padding: 1ex;
    background: transparent;
}
table.timetable_setup th {
    text-align: left;
}


/* XXX This style should be merged with table.acl in schoolbell style.css */
table.container {
    border: none;
    border-collapse: collapse;
    width: 99%;
    border-left: 1px solid #bab5ab;
    margin-top: 1ex;
}

table.container th {
    text-align: left;
    border-right: 1px solid #bab5ab;
    border-top: 1px solid #bab5ab;
    border-bottom: 1px solid #bab5ab;
    padding: 2px 0.5em;
}

table.container td {
    text-align: left;
    border-right: 1px solid #bab5ab;
    border-bottom: 1px solid #bab5ab;
    padding: 2px 0.5em;
}

.header {
    background-color: #DDD;
}

/* Styles for the zc tables */

table.data {
    border-collapse: collapse;
}

tr.odd {
    background-color: transparent;
}

tr.even {
    background-color: #EAE8E3;
}

div.field-odd {
   background-color: transparent;
}

div.field-even {
    background-color: #EAE8E3;
}

table.data .zc-table-sortable {
    cursor: pointer;
}

/*Styles for actions bar */

#dropdowncontainer {
  position: absolute;
  left: 0px;
  clear: left;
  z-index: 100;
}

#dropdowncontainer tr td {
  vertical-align: top;
}

/*For the messed up schooltool action viewlets*/
#dropdowncontainer tr td li{
  list-style: none;
  font-size: 1em;
  padding-right: .5ex;
  padding-left: .5ex;
  background: #ddd;
  border: 1px solid #aaa;
  line-height:1.2em;
  height:1.2em;
}

#dropdowncontainer tr td li a{
  font-family: "Trebuchet MS", Sans-Serif;
  text-decoration: none;
  display: block;
}

#dropdowncontainer tr td div{
  border: 1px solid #aaa;
}

div .singleton .title{
  font-size: 1em;
  padding-right: .5ex;
  padding-left: .5ex;
  background: #ddd;
}

div .singleton:hover .title{
  background:#eee;
}

div .singleton a.title{
  font-family: "Trebuchet MS", Sans-Serif;
  text-decoration: none;
  display: block;
  }

div .dropdown .title{
  cursor: pointer;
  font-size: 1em;
  padding-right: .5ex;
  padding-left: 15px;
  background: #ddd url('++resource++rightarrow.png') no-repeat center left;
}

div .dropdown:hover .title{
  background: #ddd url('++resource++downarrow.png') no-repeat center left;
}

div .dropdown ul{
  padding: 0px;
  margin: 0px;
  text-indent: 0px;
}

div .dropdown li{
  margin: 0px;
  border: 0px;
  padding: 0px;
}

div .dropdown li a{
  background: #eee;
  border: 1px solid #eee;
  border-left: 0px;
  border-right: 0px;
  line-height:18px;
  color:#304D6B;
  text-decoration:none;
  width: auto;
  padding-left: 0px;
  padding-right: 0px;
  font-size: x-small;
}

div .dropdown li a:hover{
  background: #7590ae;
  border-top: 1px #314e6c solid;
  border-bottom: 1px #314e6c solid;
  color: #fff !important;
}

div .dropdown a {
  background: #eee;
  border: 1px solid #eee;
  border-left: 0px;
  border-right: 0px;
  display:block;
  line-height:18px;
  color:#304D6B;
  text-decoration:none;
  width: auto;
  padding-left: .5ex;
  padding-right: .5ex;
  font-size: x-small;
}

div .dropdown a:hover {
  background: #7590ae;
  border-top: 1px #314e6c solid;
  border-bottom: 1px #314e6c solid;
  color: #fff !important;
}

#dropdowncontainer .expanded {
  height: auto;
  background:#eee;
}

#dropdowncontainer .contracted {
  width: auto;
  line-height:1.2em;
  height:1.2em;
  overflow:hidden;
  text-align:left;
  z-index:100;
  font-weight: bold;
}

/* Terms for Timetables */
.term0 { background-color: transparent; }
.term1 { background-color: #9db8d2; }
.term2 { background-color: #eed680; }
.term3 { background-color: #c5d2c8; }
.term4 { background-color: #e0b6af; }
.term5 { background-color: #ada7c8; }
.term6 { background-color: #7590ae; }

#header .next_schoolyear {
  color: #000000;
  font-size: small;
}

#header .active_schoolyear {
}

/* colors for table cells and warning message */
td.changed_bg { background-color: #90ee90; }
td.error_bg { background-color: #f66; }
td.warning_bg { background-color: yellow; }
div.warning_bg { background-color: yellow; }

/* section_linkage view */
table.all_terms_header td {
    padding-right: 40px;
}
table.all_terms_table th {
    background-color: #aea79f;
    color: #ffffff;
    font-size: 12px;
    line-height: 16px;
    padding-top: 4px;
    padding-bottom: 4px;
    padding-left: 8px;
    padding-right: 8px;
}
table.all_terms_table td {
    width: 220px;
    vertical-align: top;
    color: #333333;
    font-size: 12px;
    line-height: 16px;
    padding-top: 4px;
    padding-bottom: 4px;
    border-bottom: 1px dotted #cccccc;
}
table.linkage_actions td {
    border-bottom: none;
}
table.link_help td {
    width: 600px;
}
tr.link_error td {
    color: red;
    padding-top: 10px;
    padding-bottom: 10px;
}

