/* Hide the navigation */
#column-left
{
  display: none;
}

#header
{
  display: none;
}

#footer
{
  display: none;
}

/* make links black */
A
{
  color: #000000;
}

/* fix up the batching */
.batch-navigation
{
  text-align: center;
  margin-top: 1em;
  margin-bottom: 1em;
}

.batch-navigation > .previous
{
  display: none;
}

.batch-navigation > .next
{
  display: none;
}

.batch-navigation > .batch-extra-navigation
{
  display: none;
}

.batch-navigation .current
{
  font-weight: bold;
  font-size: 130%;
}

/* Make zc.table tables look nice */

.sort-indicator
{
  display: none;
}

.data
{
  border-width: 1px 1px 1px 1px;
  border-style: solid;
  margin-top: 1em;
  margin-bottom: 1em;
}

.data TD
{
  border-width: 1px 1px 1px 1px;
  border-style: solid;
}

.data INPUT
{
  display: none;
}

.data A
{
  margin: 0.3em;
  text-decoration: none;
}
