/*
Copyright (c) 2003-2011, CKSource - Frederico Knabben. All rights reserved.
For licensing, see LICENSE.html or http://ckeditor.com/license
*/

body
{
	/* Font */
	font-family: Ubuntu, Arial, Verdana, sans-serif;
	font-size: 12px;

	/* Text color */
	color: #222;

	/* Remove the background color to make it transparent */
	background-color: #fff;
}

ol,ul,dl
{
	/* IE7: reset rtl list margin. (#7334) */
	*margin-right:0px;
	/* preserved spaces for list items with text direction other than the list. (#6249,#8049)*/
    padding:0 40px;
}
