/* LOCKED */
.tree-classic .locked li a { color:gray; }
/* DOTS */
.tree-classic ul { background-position:6px 1px; background-repeat:repeat-y; background-image:url(data:image/gif;base64,R0lGODlhAgACAIAAAB4dGf///yH5BAEAAAEALAAAAAACAAIAAAICRF4AOw==); _background-image:url("dot_for_ie.gif"); *background-image:url("dot_for_ie.gif"); }
.tree-classic li { background-position:-64px -16px; background-repeat:no-repeat; background-image:url("icons.png"); }
/* NO DOTS */
.tree-classic .no_dots, .tree-classic .no_dots ul { background:transparent; }
.tree-classic .no_dots li.leaf { background-image:none; background-color:transparent; }
/* OPEN or CLOSED */
.tree-classic li.open { background:url("icons.png") -32px -48px no-repeat; }
.tree-classic li.closed, #jstree-dragged.tree-classic li li.open { background:url("icons.png") -48px -32px no-repeat; }
#jstree-marker { background-image:url("icons.png"); }
/* DEFAULT, HOVER, CLICKED, LOADING STATES */
.tree-classic li a:hover, .tree-classic li a.hover, .tree-classic li span { /*background: #e7f4f9; border:1px solid #d8f0fa; padding:0px 3px 0px 3px;*/ }
.tree-classic li a.clicked, .tree-classic li a.clicked:hover, .tree-classic li span.clicked { background: navy; border:1px solid navy; color:white; padding:0px 3px 0px 3px; }
/* ICONS */
.tree-classic ins { background-image:url("icons.png"); background-position:0 0; background-repeat:no-repeat; }
.tree-classic ul li a.loading ins { background-image:url("throbber.gif") !important; background-position:0 0 !important; } /* UL is added to make selector stronger */
.tree-classic li.closed a ins, .tree-classic li.leaf a ins { background-position:0 0px; } /* UL is added to make selector stronger */
.tree-classic ul li a.loading ins { background-image:url("throbber.gif"); background-position:0 0; } /* UL is added to make selector stronger */
.tree-classic li a ins.forbidden { background-position:-16px -16px; }
.tree-classic .locked li a ins { background-position:0 -48px; }
.tree-classic li span ins { background-position:-16px 0; }
#jstree-dragged.tree-classic ins { background:url("icons.png") -16px -32px no-repeat; }
#jstree-dragged.tree-classic ins.forbidden { background:url("icons.png") -16px -16px no-repeat; }

/* CONTEXT MENU */
.tree-classic-context a ins { background-image:url("icons.png"); background-repeat:no-repeat; background-position:-64px -64px; }
.tree-classic-context a ins.create { background-position:0 -16px; }
.tree-classic-context a ins.rename { background-position:-16px 0px; }
.tree-classic-context a ins.remove { background-position:-32px -16px; }